@@ -3,14 +3,13 @@
 #
 # Author: Albert Wu
 #
-# Compiler for 61A TA repo. Supports template inheritance and embedded
-# Python.
-#
+# Supports template inheritance and embedded Python.
 ######################################################################
 
 import argparse
 import os
 import re
+import traceback
 import templar.link as link
 from templar import log
 from templar.markdown import Markdown
@@ -237,7 +236,7 @@ class Frame:
         try:
             return eval(variable)
         except:
-            return ''
+            raise KeyError(variable)
 
     def __contains__(self, variable):
         variable = variable.replace('\n', ' ').strip()
@@ -254,7 +253,10 @@ def evaluate(expression, attrs):
     else:
         try:
             return eval(expression, {}, attrs)
-        except:
+        except Exception as e:
+            log.warn('"{}" caused {}: {}'.format(expression,
+                e.__class__.__name__, e))
+            traceback.print_exc()
             return ''
 
 
@@ -449,7 +451,7 @@ def file_write(filename, text):
     This function exists to make test injection easier.
     """
     dirname = os.path.dirname(filename)
-    if not os.path.exists(dirname):
+    if dirname and not os.path.exists(dirname):
         os.makedirs(dirname)
     with open(filename, 'w') as f:
         f.write(text)
