@@ -2,15 +2,16 @@ import argparse
 import os
 import re
 import sys
+import textwrap
 from collections import OrderedDict
-
 from templar.markdown import convert
+from templar import log
 
 ##############
 # Public API #
 ##############
 
-def link(text):
+def link(filepath):
     """Link the text based on <include> tags.
 
     An include tag should have the following syntax:
@@ -56,7 +57,8 @@ def link(text):
     text -- str; the text after resolving all include tags. Any block
             tags that are left over are preserved in text.
     """
-    return substitute_links(text, OrderedDict(), '')
+    return substitute_links(file_read(filepath), OrderedDict(),
+            os.path.dirname(filepath))
 
 def retrieve_blocks(text):
     """Strip block tags from text and return a mapping of block
@@ -125,9 +127,6 @@ def scrape_headers(text, builder):
 # Linker #
 ##########
 
-def add_whitespace(text, whitespace):
-    return whitespace + ('\n' + whitespace).join(line for line in text.split('\n'))
-
 re_include = re.compile(r"""
 (?:(?<=\n)|(?<=\A))
 ([ \t]*)      # \1 is leading whitespace
@@ -164,12 +163,17 @@ def substitute_links(text, cache, base_dir):
         regex = match.group(3)
         if not file_exists(filename):
             filename = os.path.join(base_dir, filename)
+            if not file_exists(filename):
+                log.warn("Could not find file " + match.group(2) \
+                        + " or " + filename)
+                return ''
+
         text = retrieve_and_link(filename, cache)
         if not regex:
             result = cache[filename + ':all']
         else:
             result = resolve_include_regex(regex, cache, filename)
-        return add_whitespace(result, match.group(1))
+        return textwrap.indent(result, match.group(1))
     return re_include.sub(link_sub, text)
 
 def retrieve_and_link(filename, cache):
@@ -306,6 +310,9 @@ def file_write(filename, text):
     NOTE:
     This function exists to make test injection easier.
     """
+    dirname = os.path.dirname(filename)
+    if not os.path.exists(dirname):
+        os.makedirs(dirname)
     with open(filename, 'w') as f:
         f.write(text)
 
@@ -328,14 +335,12 @@ def cmd_options(parser):
 
 def main(args, configs):
     if not os.path.exists(args.source):
-        print('File ' + args.source + ' does not exist.',
-              file=sys.stderr)
+        log.warn('File ' + args.source + ' does not exist.')
         exit(1)
     elif not os.path.isfile(args.source):
-        print(args.source + ' is not a valid file',
-              file=sys.stderr)
+        log.warn(args.source + ' is not a valid file')
         exit(1)
-    result = link(file_read(args.source))
+    result = link(args.source)
     if args.markdown:
         result = convert(result)
     result = substitutions(result,
@@ -344,14 +349,12 @@ def main(args, configs):
     result, cache = retrieve_blocks(result)
     if args.destination:
         file_write(args.destination, result)
-        print('Result can be found in ' + args.destination)
+        log.info('Result can be found in ' + args.destination)
     else:
-        if not args.quiet:
-            print('--- BEGIN RESULT ---')
-        print(result)
-        if not args.quiet:
-            print('--- END RESULT ---')
+        log.log('--- BEGIN RESULT ---', args.quiet)
+        log.log(result)
+        log.log('--- END RESULT ---', args.quiet)
 
 if __name__ == '__main__':
-    print('Usage: python3 __main__.py link ...')
+    log.log('Usage: python3 __main__.py link ...')
 
